"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = matcher;
exports.wildcard = exports.symbol = exports.string = exports.predicate = exports.array = void 0;

var _utils =
/*#__PURE__*/
require("./utils");

var array = function array(patterns) {
  return function (input) {
    return patterns.some(function (p) {
      return matcher(p)(input);
    });
  };
};

exports.array = array;

var predicate = function predicate(_predicate) {
  return function (input) {
    return _predicate(input);
  };
};

exports.predicate = predicate;

var string = function string(pattern) {
  return function (input) {
    return input.type === String(pattern);
  };
};

exports.string = string;

var symbol = function symbol(pattern) {
  return function (input) {
    return input.type === pattern;
  };
};

exports.symbol = symbol;

var wildcard = function wildcard() {
  return _utils.kTrue;
};

exports.wildcard = wildcard;

function matcher(pattern) {
  // prettier-ignore
  var matcherCreator = pattern === '*' ? wildcard : _utils.is.string(pattern) ? string : _utils.is.array(pattern) ? array : _utils.is.stringableFunc(pattern) ? string : _utils.is.func(pattern) ? predicate : _utils.is.symbol(pattern) ? symbol : null;

  if (matcherCreator === null) {
    throw new Error("invalid pattern: " + pattern);
  }

  return matcherCreator(pattern);
}