"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.safeName = safeName;
exports.default = fsmIterator;
exports.qEnd = void 0;

var _utils =
/*#__PURE__*/
require("../utils");

var done = {
  done: true,
  value: undefined
};
var qEnd = {};
exports.qEnd = qEnd;

function safeName(patternOrChannel) {
  if (_utils.is.channel(patternOrChannel)) {
    return 'channel';
  }

  if (_utils.is.stringableFunc(patternOrChannel)) {
    return String(patternOrChannel);
  }

  if (_utils.is.func(patternOrChannel)) {
    return patternOrChannel.name;
  }

  return String(patternOrChannel);
}

function fsmIterator(fsm, q0, name) {
  var updateState,
      qNext = q0;

  function next(arg, error) {
    if (qNext === qEnd) {
      return done;
    }

    if (error) {
      qNext = qEnd;
      throw error;
    } else {
      updateState && updateState(arg);

      var _fsm$qNext = fsm[qNext](),
          q = _fsm$qNext[0],
          output = _fsm$qNext[1],
          _updateState = _fsm$qNext[2];

      qNext = q;
      updateState = _updateState;
      return qNext === qEnd ? done : output;
    }
  }

  return (0, _utils.makeIterator)(next, function (error) {
    return next(null, error);
  }, name);
}